@@ -18,6 +18,7 @@
 
 package org.apache.gobblin.cluster;
 
+import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
@@ -47,12 +48,12 @@ import org.apache.helix.task.WorkflowConfig;
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import com.google.common.base.Throwables;
+import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Lists;
 import com.google.common.eventbus.EventBus;
 import com.google.common.util.concurrent.MoreExecutors;
 import com.typesafe.config.Config;
 
-import javax.annotation.Nonnull;
 import lombok.Getter;
 import lombok.extern.slf4j.Slf4j;
 
@@ -476,19 +477,8 @@ public class GobblinHelixMultiManager implements StandardMetricsBridge {
     }
   }
 
-  @Nonnull
   @Override
-  public MetricContext getMetricContext() {
-    return this.metricContext;
-  }
-
-  @Override
-  public boolean isInstrumentationEnabled() {
-    return true;
-  }
-
-  @Override
-  public StandardMetrics getStandardMetrics() {
-    return this.metrics;
+  public Collection<StandardMetrics> getStandardMetricsCollection() {
+    return ImmutableList.of(this.metrics);
   }
 }
