@@ -25,8 +25,8 @@ import org.apache.helix.HelixManager;
 import org.apache.helix.manager.zk.ZKHelixManager;
 import org.apache.helix.model.HelixConfigScope;
 import org.apache.helix.task.JobConfig;
-import org.apache.helix.task.TargetState;
 import org.apache.helix.task.TaskDriver;
+import org.apache.helix.task.TaskState;
 import org.apache.helix.task.TaskUtil;
 import org.apache.helix.task.Workflow;
 import org.apache.helix.task.WorkflowConfig;
@@ -36,6 +36,11 @@ import org.apache.helix.tools.ClusterSetup;
 import lombok.extern.slf4j.Slf4j;
 
 import org.apache.gobblin.annotation.Alpha;
+import org.apache.gobblin.runtime.JobException;
+import org.apache.gobblin.runtime.listeners.JobListener;
+
+import static org.apache.helix.task.TaskState.STOPPED;
+
 
 /**
  * A utility class for working with Gobblin on Helix.
@@ -115,15 +120,26 @@ public class HelixUtils {
     // start the workflow
     helixTaskDriver.start(workFlow);
     log.info("Created a work flow {}", workFlowName);
+    WorkflowContext workflowContext = TaskDriver.getWorkflowContext(helixManager, workFlowName);
+
+    // If the helix job is deleted from some other thread or a completely external process,
+    // method waitJobCompletion() needs to differentiate between the cases where
+    // 1) workflowContext did not get initialized ever, in which case we need to keep waiting, or
+    // 2) it did get initialized but deleted soon after, in which case we should stop waiting
+    // To overcome this issue, we wait here till workflowContext gets initialized
+
+    while (workflowContext == null || workflowContext.getJobState(TaskUtil.getNamespacedJobName(workFlowName, jobName)) == null) {
+      workflowContext = TaskDriver.getWorkflowContext(helixManager, workFlowName);
+      Thread.sleep(1000);
+      log.info("Waiting for work flow initialization.");
+    }
+    log.info("Work flow {} initialized", workFlowName);
   }
 
-  public static void waitJobCompletion(
-      HelixManager helixManager,
-      String workFlowName,
-      String jobName,
+  static void waitJobCompletion(HelixManager helixManager, String workFlowName, String jobName,
       Optional<Long> timeoutInSeconds) throws InterruptedException, TimeoutException {
 
-    log.info("Waiting for job to complete...");
+    log.info("Waiting for job {} to complete...", jobName);
     long endTime = 0;
     if (timeoutInSeconds.isPresent()) {
       endTime = System.currentTimeMillis() + timeoutInSeconds.get() * 1000;
@@ -132,16 +148,69 @@ public class HelixUtils {
     while (!timeoutInSeconds.isPresent() || System.currentTimeMillis() <= endTime) {
       WorkflowContext workflowContext = TaskDriver.getWorkflowContext(helixManager, workFlowName);
       if (workflowContext != null) {
-        org.apache.helix.task.TaskState helixJobState = workflowContext.getJobState(TaskUtil.getNamespacedJobName(workFlowName, jobName));
-        if (helixJobState == org.apache.helix.task.TaskState.COMPLETED ||
-            helixJobState == org.apache.helix.task.TaskState.FAILED ||
-            helixJobState == org.apache.helix.task.TaskState.STOPPED) {
+        TaskState jobState = workflowContext.getJobState(TaskUtil.getNamespacedJobName(workFlowName, jobName));
+        switch (jobState) {
+          case STOPPED:
+            // user requested cancellation, which is executed by executeCancellation()
+            log.info("Job {} is cancelled, it will be deleted now.", jobName);
+            HelixUtils.deleteStoppedHelixJob(helixManager, workFlowName, jobName);
+            return;
+          case FAILED:
+          case COMPLETED:
           return;
+          default:
+            log.info("Waiting for job {} to complete...", jobName);
+            Thread.sleep(1000);
         }
+      } else {
+        // We have waited for WorkflowContext to get initialized,
+        // so it is found null here, it must have been deleted in job cancellation process.
+        log.info("WorkflowContext not found. Job is probably cancelled.");
+        return;
       }
-      Thread.sleep(1000);
     }
 
     throw new TimeoutException("task driver wait time [" + timeoutInSeconds + " sec] is expired.");
   }
-}
+
+  static void handleJobTimeout(String workFlowName, String jobName, HelixManager helixManager, Object jobLauncher,
+      JobListener jobListener) throws InterruptedException {
+    try {
+      log.warn("Timeout occurred for job launcher {} with job {}", jobLauncher.getClass(), jobName);
+      if (jobLauncher instanceof GobblinHelixJobLauncher) {
+        ((GobblinHelixJobLauncher) jobLauncher).cancelJob(jobListener);
+      } else if (jobLauncher instanceof GobblinHelixDistributeJobExecutionLauncher) {
+        ((GobblinHelixDistributeJobExecutionLauncher) jobLauncher).cancel();
+      }
+    } catch (JobException e) {
+      throw new RuntimeException("Unable to cancel job " + jobName + ": ", e);
+    }
+    // TODO : fix this when HELIX-1180 is completed
+    // We should not be deleting a workflow explicitly.
+    // Workflow state should be set to a final state, which will remove it automatically because expiry time is set.
+    // After that, all delete calls can be replaced by something like HelixUtils.setStateToFinal();
+    HelixUtils.deleteStoppedHelixJob(helixManager, workFlowName, jobName);
+    log.info("Stopped and deleted the workflow {}", workFlowName);
+  }
+
+  /**
+   * Deletes the stopped Helix Workflow.
+   * Caller should stop the Workflow before calling this method.
+   * @param helixManager helix manager
+   * @param workFlowName workflow needed to be deleted
+   * @param jobName helix job name
+   * @throws InterruptedException
+   */
+  private static void deleteStoppedHelixJob(HelixManager helixManager, String workFlowName, String jobName)
+      throws InterruptedException {
+    WorkflowContext workflowContext = TaskDriver.getWorkflowContext(helixManager, workFlowName);
+    while (workflowContext.getJobState(TaskUtil.getNamespacedJobName(workFlowName, jobName)) != STOPPED) {
+      log.info("Waiting for job {} to stop...", jobName);
+      workflowContext = TaskDriver.getWorkflowContext(helixManager, workFlowName);
+      Thread.sleep(1000);
+    }
+    // deleting the entire workflow, as one workflow contains only one job
+    new TaskDriver(helixManager).deleteAndWaitForCompletion(workFlowName, 10000L);
+    log.info("Workflow deleted.");
+  }
+}
\ No newline at end of file
