@@ -765,18 +765,17 @@ public class HiveCopyEntityHelper {
     return this.targetFs;
   }
 
-  /**
-   * Set the source and destination datasets of a copyable file
-   */
-  void setCopyableFileDatasets(CopyableFile copyableFile) {
+  DatasetDescriptor getSourceDataset() {
     String sourceTable = dataset.getTable().getDbName() + "." + dataset.getTable().getTableName();
-    DatasetDescriptor source = new DatasetDescriptor(DatasetConstants.PLATFORM_HIVE, sourceTable);
-    source.addMetadata(DatasetConstants.FS_URI, dataset.getFs().getUri().toString());
-    copyableFile.setSourceDataset(source);
+    DatasetDescriptor sourceDataset = new DatasetDescriptor(DatasetConstants.PLATFORM_HIVE, sourceTable);
+    sourceDataset.addMetadata(DatasetConstants.FS_URI, dataset.getFs().getUri().toString());
+    return sourceDataset;
+  }
 
+  DatasetDescriptor getDestinationDataset() {
     String destinationTable = this.getTargetDatabase() + "." + this.getTargetTable();
-    DatasetDescriptor destination = new DatasetDescriptor(DatasetConstants.PLATFORM_HIVE, destinationTable);
-    destination.addMetadata(DatasetConstants.FS_URI, this.getTargetFs().getUri().toString());
-    copyableFile.setDestinationDataset(destination);
+    DatasetDescriptor destinationDataset = new DatasetDescriptor(DatasetConstants.PLATFORM_HIVE, destinationTable);
+    destinationDataset.addMetadata(DatasetConstants.FS_URI, this.getTargetFs().getUri().toString());
+    return destinationDataset;
   }
 }
