@@ -27,10 +27,14 @@ import java.util.List;
 import java.util.Map;
 import java.util.concurrent.atomic.AtomicLong;
 
+import lombok.extern.slf4j.Slf4j;
+
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FSDataInputStream;
+import org.apache.hadoop.fs.FileContext;
 import org.apache.hadoop.fs.FileStatus;
 import org.apache.hadoop.fs.FileSystem;
+import org.apache.hadoop.fs.Options;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.fs.permission.FsAction;
 import org.apache.hadoop.fs.permission.FsPermission;
@@ -40,9 +44,6 @@ import com.google.common.base.Optional;
 import com.google.common.base.Predicate;
 import com.google.common.base.Strings;
 import com.google.common.collect.Iterators;
-import com.google.common.io.Closer;
-
-import lombok.extern.slf4j.Slf4j;
 
 import org.apache.gobblin.broker.EmptyKey;
 import org.apache.gobblin.broker.gobblin_scopes.GobblinScopeTypes;
@@ -61,10 +62,9 @@ import org.apache.gobblin.data.management.copy.CopyableDatasetMetadata;
 import org.apache.gobblin.data.management.copy.CopyableFile;
 import org.apache.gobblin.data.management.copy.FileAwareInputStream;
 import org.apache.gobblin.data.management.copy.OwnerAndPermission;
-import org.apache.gobblin.data.management.copy.PreserveAttributes;
 import org.apache.gobblin.data.management.copy.recovery.RecoveryHelper;
+import org.apache.gobblin.data.management.copy.splitter.DistcpFileSplitter;
 import org.apache.gobblin.instrumented.writer.InstrumentedDataWriter;
-import org.apache.gobblin.metrics.event.sla.SlaEventKeys;
 import org.apache.gobblin.state.ConstructState;
 import org.apache.gobblin.util.FileListUtils;
 import org.apache.gobblin.util.FinalState;
@@ -86,6 +86,8 @@ public class FileAwareInputStreamDataWriter extends InstrumentedDataWriter<FileA
   public static final String GOBBLIN_COPY_BYTES_COPIED_METER = "gobblin.copy.bytesCopiedMeter";
   public static final String GOBBLIN_COPY_CHECK_FILESIZE = "gobblin.copy.checkFileSize";
   public static final boolean DEFAULT_GOBBLIN_COPY_CHECK_FILESIZE = false;
+  public static final String GOBBLIN_COPY_TASK_OVERWRITE_ON_COMMIT = "gobblin.copy.task.overwrite.on.commit";
+  public static final boolean DEFAULT_GOBBLIN_COPY_TASK_OVERWRITE_ON_COMMIT = false;
 
   protected final AtomicLong bytesWritten = new AtomicLong();
   protected final AtomicLong filesWritten = new AtomicLong();
@@ -99,6 +101,8 @@ public class FileAwareInputStreamDataWriter extends InstrumentedDataWriter<FileA
   protected final SharedResourcesBroker<GobblinScopeTypes> taskBroker;
   protected final int bufferSize;
   private final boolean checkFileSize;
+  private final Options.Rename renameOptions;
+  private final FileContext fileContext;
 
   protected final Meter copySpeedMeter;
 
@@ -127,11 +131,15 @@ public class FileAwareInputStreamDataWriter extends InstrumentedDataWriter<FileA
 
     this.writerAttemptIdOptional = Optional.fromNullable(writerAttemptId);
 
-    String uri = this.state.getProp(
+    String uriStr = this.state.getProp(
         ForkOperatorUtils.getPropertyNameForBranch(ConfigurationKeys.WRITER_FILE_SYSTEM_URI, numBranches, branchId),
         ConfigurationKeys.LOCAL_FS_URI);
 
-    this.fs = FileSystem.get(URI.create(uri), WriterUtils.getFsConfiguration(state));
+    Configuration conf = WriterUtils.getFsConfiguration(state);
+    URI uri = URI.create(uriStr);
+    this.fs = FileSystem.get(uri, conf);
+    this.fileContext = FileContext.getFileContext(uri, conf);
+
     this.stagingDir = this.writerAttemptIdOptional.isPresent() ? WriterUtils
         .getWriterStagingDir(state, numBranches, branchId, this.writerAttemptIdOptional.get())
         : WriterUtils.getWriterStagingDir(state, numBranches, branchId);
@@ -148,6 +156,12 @@ public class FileAwareInputStreamDataWriter extends InstrumentedDataWriter<FileA
         .getConfigForBranch(EncryptionConfigParser.EntityType.WRITER, this.state, numBranches, branchId);
 
     this.checkFileSize = state.getPropAsBoolean(GOBBLIN_COPY_CHECK_FILESIZE, DEFAULT_GOBBLIN_COPY_CHECK_FILESIZE);
+    boolean taskOverwriteOnCommit = state.getPropAsBoolean(GOBBLIN_COPY_TASK_OVERWRITE_ON_COMMIT, DEFAULT_GOBBLIN_COPY_TASK_OVERWRITE_ON_COMMIT);
+    if (taskOverwriteOnCommit) {
+      this.renameOptions = Options.Rename.OVERWRITE;
+    } else {
+      this.renameOptions = Options.Rename.NONE;
+    }
   }
 
   public FileAwareInputStreamDataWriter(State state, int numBranches, int branchId)
@@ -169,7 +183,7 @@ public class FileAwareInputStreamDataWriter extends InstrumentedDataWriter<FileA
     }
     this.actualProcessedCopyableFile = Optional.of(copyableFile);
     this.fs.mkdirs(stagingFile.getParent());
-    writeImpl(fileAwareInputStream.getInputStream(), stagingFile, copyableFile);
+    writeImpl(fileAwareInputStream.getInputStream(), stagingFile, copyableFile, fileAwareInputStream);
     this.filesWritten.incrementAndGet();
   }
 
@@ -186,17 +200,31 @@ public class FileAwareInputStreamDataWriter extends InstrumentedDataWriter<FileA
    * @param inputStream {@link FSDataInputStream} whose contents should be written to staging path.
    * @param writeAt {@link Path} at which contents should be written.
    * @param copyableFile {@link org.apache.gobblin.data.management.copy.CopyEntity} that generated this copy operation.
+   * @param record The actual {@link FileAwareInputStream} passed to the write method.
    * @throws IOException
    */
-  protected void writeImpl(InputStream inputStream, Path writeAt, CopyableFile copyableFile)
-      throws IOException {
+  protected void writeImpl(InputStream inputStream, Path writeAt, CopyableFile copyableFile,
+      FileAwareInputStream record) throws IOException {
 
-    final short replication =
-        copyableFile.getPreserve().preserve(PreserveAttributes.Option.REPLICATION) ? copyableFile.getOrigin()
-            .getReplication() : this.fs.getDefaultReplication(writeAt);
-    final long blockSize =
-        copyableFile.getPreserve().preserve(PreserveAttributes.Option.BLOCK_SIZE) ? copyableFile.getOrigin()
-            .getBlockSize() : this.fs.getDefaultBlockSize(writeAt);
+    final short replication = copyableFile.getReplication(this.fs);
+    final long blockSize = copyableFile.getBlockSize(this.fs);
+    final long fileSize = copyableFile.getFileStatus().getLen();
+
+    long expectedBytes = fileSize;
+    Long maxBytes = null;
+    // Whether writer must write EXACTLY maxBytes.
+    boolean mustMatchMaxBytes = false;
+
+    if (record.getSplit().isPresent()) {
+      maxBytes = record.getSplit().get().getHighPosition() - record.getSplit().get().getLowPosition();
+      if (record.getSplit().get().isLastSplit()) {
+        expectedBytes = fileSize % blockSize;
+        mustMatchMaxBytes = false;
+      } else {
+        expectedBytes = maxBytes;
+        mustMatchMaxBytes = true;
+      }
+    }
 
     Predicate<FileStatus> fileStatusAttributesFilter = new Predicate<FileStatus>() {
       @Override
@@ -229,7 +257,7 @@ public class FileAwareInputStreamDataWriter extends InstrumentedDataWriter<FileA
         ThrottledInputStream throttledInputStream = throttler.throttleInputStream().inputStream(inputStream)
             .sourceURI(copyableFile.getOrigin().getPath().makeQualified(defaultFS.getUri(), defaultFS.getWorkingDirectory()).toUri())
             .targetURI(this.fs.makeQualified(writeAt).toUri()).build();
-        StreamCopier copier = new StreamCopier(throttledInputStream, os).withBufferSize(this.bufferSize);
+        StreamCopier copier = new StreamCopier(throttledInputStream, os, maxBytes).withBufferSize(this.bufferSize);
 
         log.info("File {}: Starting copy", copyableFile.getOrigin().getPath());
 
@@ -237,10 +265,9 @@ public class FileAwareInputStreamDataWriter extends InstrumentedDataWriter<FileA
           copier.withCopySpeedMeter(this.copySpeedMeter);
         }
         long numBytes = copier.copy();
-        long fileSize = copyableFile.getFileStatus().getLen();
-        if (this.checkFileSize && numBytes != fileSize) {
-          throw new IOException(String.format("Number of bytes copied doesn't match filesize for file %s.",
-              copyableFile.getOrigin().getPath()));
+        if ((this.checkFileSize || mustMatchMaxBytes) && numBytes != expectedBytes) {
+          throw new IOException(String.format("Incomplete write: expected %d, wrote %d bytes.",
+              expectedBytes, numBytes));
         }
         this.bytesWritten.addAndGet(numBytes);
         if (isInstrumentationEnabled()) {
@@ -267,6 +294,9 @@ public class FileAwareInputStreamDataWriter extends InstrumentedDataWriter<FileA
   }
 
   protected Path getStagingFilePath(CopyableFile file) {
+    if (DistcpFileSplitter.isSplitWorkUnit(this.state)) {
+      return new Path(this.stagingDir, DistcpFileSplitter.getSplit(this.state).get().getPartName());
+    }
     return new Path(this.stagingDir, file.getDestination().getName());
   }
 
@@ -281,6 +311,16 @@ public class FileAwareInputStreamDataWriter extends InstrumentedDataWriter<FileA
         PathUtils.withoutLeadingSeparator(destinationWithoutSchemeAndAuthority));
   }
 
+  public static Path getSplitOutputFilePath(CopyableFile file, Path outputDir,
+      CopyableFile.DatasetAndPartition datasetAndPartition, State workUnit) {
+    if (DistcpFileSplitter.isSplitWorkUnit(workUnit)) {
+      return new Path(getOutputFilePath(file, outputDir, datasetAndPartition).getParent(),
+          DistcpFileSplitter.getSplit(workUnit).get().getPartName());
+    } else {
+      return getOutputFilePath(file, outputDir, datasetAndPartition);
+    }
+  }
+
   public static Path getOutputDir(State state) {
     return new Path(
         state.getProp(ForkOperatorUtils.getPropertyNameForBranch(ConfigurationKeys.WRITER_OUTPUT_DIR, 1, 0)));
@@ -369,7 +409,7 @@ public class FileAwareInputStreamDataWriter extends InstrumentedDataWriter<FileA
    *
    * {@inheritDoc}
    *
-   * @see org.apache.gobblin.writer.DataWriter#commit()
+   * @see DataWriter#commit()
    */
   @Override
   public void commit()
@@ -381,8 +421,8 @@ public class FileAwareInputStreamDataWriter extends InstrumentedDataWriter<FileA
 
     CopyableFile copyableFile = this.actualProcessedCopyableFile.get();
     Path stagingFilePath = getStagingFilePath(copyableFile);
-    Path outputFilePath = getOutputFilePath(copyableFile, this.outputDir,
-        copyableFile.getDatasetAndPartition(this.copyableDatasetMetadata));
+    Path outputFilePath = getSplitOutputFilePath(copyableFile, this.outputDir,
+        copyableFile.getDatasetAndPartition(this.copyableDatasetMetadata), this.state);
 
     log.info(String.format("Committing data from %s to %s", stagingFilePath, outputFilePath));
     try {
@@ -394,11 +434,9 @@ public class FileAwareInputStreamDataWriter extends InstrumentedDataWriter<FileA
 
       ensureDirectoryExists(this.fs, outputFilePath.getParent(), ancestorOwnerAndPermissionIt);
 
-      if (!this.fs.rename(stagingFilePath, outputFilePath)) {
-        // target exists
-        throw new IOException(String.format("Could not commit file %s.", outputFilePath));
-      }
+      this.fileContext.rename(stagingFilePath, outputFilePath, renameOptions);
     } catch (IOException ioe) {
+      log.error("Could not commit file %s.", outputFilePath);
       // persist file
       this.recoveryHelper.persistFile(this.state, copyableFile, stagingFilePath);
       throw ioe;
