@@ -17,7 +17,6 @@
 
 package org.apache.gobblin.hive.metastore;
 
-import com.google.common.base.Splitter;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.util.ArrayList;
@@ -49,17 +48,23 @@ import org.slf4j.LoggerFactory;
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Optional;
 import com.google.common.base.Preconditions;
+import com.google.common.base.Splitter;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 import com.google.common.primitives.Ints;
 
 import org.apache.gobblin.annotation.Alpha;
+import org.apache.gobblin.broker.EmptyKey;
+import org.apache.gobblin.broker.SharedResourcesBrokerFactory;
+import org.apache.gobblin.broker.iface.NotConfiguredException;
 import org.apache.gobblin.configuration.State;
+import org.apache.gobblin.hive.HiveConfFactory;
 import org.apache.gobblin.hive.HiveConstants;
 import org.apache.gobblin.hive.HivePartition;
 import org.apache.gobblin.hive.HiveRegistrationUnit;
 import org.apache.gobblin.hive.HiveRegistrationUnit.Column;
 import org.apache.gobblin.hive.HiveTable;
+import org.apache.gobblin.hive.SharedHiveConfKey;
 
 
 /**
@@ -370,15 +375,19 @@ public class HiveMetaStoreUtils {
     }
 
     String serde = serdeClass.get();
-    HiveConf hiveConf = new HiveConf();
-
+    HiveConf hiveConf;
     Deserializer deserializer;
     try {
+      hiveConf = SharedResourcesBrokerFactory
+        .getImplicitBroker().getSharedResource(new HiveConfFactory<>(), SharedHiveConfKey.INSTANCE);
       deserializer =
           ReflectionUtils.newInstance(hiveConf.getClassByName(serde).asSubclass(Deserializer.class), hiveConf);
     } catch (ClassNotFoundException e) {
       LOG.warn("Serde class " + serde + " not found!", e);
       return null;
+    } catch (NotConfiguredException nce) {
+      LOG.error("Implicit broker is not configured properly", nce);
+      return null;
     }
 
     Properties props = new Properties();
