@@ -24,6 +24,8 @@ import java.util.List;
 import java.util.Map;
 import java.util.Properties;
 import java.util.concurrent.TimeUnit;
+
+import com.google.common.collect.Maps;
 import javax.annotation.Nonnull;
 
 import com.codahale.metrics.Meter;
@@ -33,12 +35,16 @@ import com.google.common.base.Optional;
 import com.typesafe.config.Config;
 
 import org.apache.gobblin.annotation.Alpha;
+import org.apache.gobblin.configuration.ConfigurationKeys;
 import org.apache.gobblin.instrumented.Instrumented;
 import org.apache.gobblin.instrumented.Instrumentable;
 import org.apache.gobblin.metrics.MetricContext;
 import org.apache.gobblin.metrics.Tag;
+import org.apache.gobblin.metrics.event.EventSubmitter;
+import org.apache.gobblin.metrics.event.TimingEvent;
 import org.apache.gobblin.runtime.api.FlowSpec;
-import org.apache.gobblin.runtime.api.SpecCompiler;
+import org.apache.gobblin.service.modules.flow.SpecCompiler;
+import org.apache.gobblin.runtime.api.JobSpec;
 import org.apache.gobblin.runtime.api.TopologySpec;
 import org.apache.gobblin.runtime.api.Spec;
 import org.apache.gobblin.runtime.api.SpecCatalogListener;
@@ -46,6 +52,8 @@ import org.apache.gobblin.runtime.spec_catalog.TopologyCatalog;
 import org.apache.gobblin.service.ServiceConfigKeys;
 import org.apache.gobblin.service.ServiceMetricNames;
 import org.apache.gobblin.service.modules.flow.IdentityFlowToJobSpecCompiler;
+import org.apache.gobblin.service.modules.flowgraph.Dag;
+import org.apache.gobblin.service.modules.spec.JobExecutionPlan;
 import org.apache.gobblin.util.ClassAliasResolver;
 import org.apache.gobblin.util.ConfigUtils;
 import org.slf4j.LoggerFactory;
@@ -67,8 +75,10 @@ public class Orchestrator implements SpecCatalogListener, Instrumentable {
   protected final Logger _log;
   protected final SpecCompiler specCompiler;
   protected final Optional<TopologyCatalog> topologyCatalog;
+  protected final Optional<DagManager> dagManager;
 
   protected final MetricContext metricContext;
+  protected final Optional<EventSubmitter> eventSubmitter;
   @Getter
   private Optional<Meter> flowOrchestrationSuccessFulMeter;
   @Getter
@@ -78,23 +88,12 @@ public class Orchestrator implements SpecCatalogListener, Instrumentable {
 
   private final ClassAliasResolver<SpecCompiler> aliasResolver;
 
-  public Orchestrator(Config config, Optional<TopologyCatalog> topologyCatalog, Optional<Logger> log, boolean instrumentationEnabled) {
+  public Orchestrator(Config config, Optional<TopologyCatalog> topologyCatalog, Optional<DagManager> dagManager, Optional<Logger> log,
+      boolean instrumentationEnabled) {
     _log = log.isPresent() ? log.get() : LoggerFactory.getLogger(getClass());
-    if (instrumentationEnabled) {
-      this.metricContext = Instrumented.getMetricContext(ConfigUtils.configToState(config), IdentityFlowToJobSpecCompiler.class);
-      this.flowOrchestrationSuccessFulMeter = Optional.of(this.metricContext.meter(ServiceMetricNames.FLOW_ORCHESTRATION_SUCCESSFUL_METER));
-      this.flowOrchestrationFailedMeter = Optional.of(this.metricContext.meter(ServiceMetricNames.FLOW_ORCHESTRATION_FAILED_METER));
-      this.flowOrchestrationTimer = Optional.<Timer>of(this.metricContext.timer(ServiceMetricNames.FLOW_ORCHESTRATION_TIMER));
-    }
-    else {
-      this.metricContext = null;
-      this.flowOrchestrationSuccessFulMeter = Optional.absent();
-      this.flowOrchestrationFailedMeter = Optional.absent();
-      this.flowOrchestrationTimer = Optional.absent();
-    }
-
     this.aliasResolver = new ClassAliasResolver<>(SpecCompiler.class);
     this.topologyCatalog = topologyCatalog;
+    this.dagManager = dagManager;
     try {
       String specCompilerClassName = ServiceConfigKeys.DEFAULT_GOBBLIN_SERVICE_FLOWCOMPILER_CLASS;
       if (config.hasPath(ServiceConfigKeys.GOBBLIN_SERVICE_FLOWCOMPILER_CLASS_KEY)) {
@@ -108,27 +107,41 @@ public class Orchestrator implements SpecCatalogListener, Instrumentable {
         | ClassNotFoundException e) {
       throw new RuntimeException(e);
     }
+
+    if (instrumentationEnabled) {
+      this.metricContext = Instrumented.getMetricContext(ConfigUtils.configToState(config), this.specCompiler.getClass());
+      this.flowOrchestrationSuccessFulMeter = Optional.of(this.metricContext.meter(ServiceMetricNames.FLOW_ORCHESTRATION_SUCCESSFUL_METER));
+      this.flowOrchestrationFailedMeter = Optional.of(this.metricContext.meter(ServiceMetricNames.FLOW_ORCHESTRATION_FAILED_METER));
+      this.flowOrchestrationTimer = Optional.of(this.metricContext.timer(ServiceMetricNames.FLOW_ORCHESTRATION_TIMER));
+      this.eventSubmitter = Optional.of(new EventSubmitter.Builder(this.metricContext, "org.apache.gobblin.service").build());
+    } else {
+      this.metricContext = null;
+      this.flowOrchestrationSuccessFulMeter = Optional.absent();
+      this.flowOrchestrationFailedMeter = Optional.absent();
+      this.flowOrchestrationTimer = Optional.absent();
+      this.eventSubmitter = Optional.absent();
+    }
   }
 
-  public Orchestrator(Config config, Optional<TopologyCatalog> topologyCatalog, Optional<Logger> log) {
-    this(config, topologyCatalog, log, true);
+  public Orchestrator(Config config, Optional<TopologyCatalog> topologyCatalog, Optional<DagManager> dagManager, Optional<Logger> log) {
+    this(config, topologyCatalog, dagManager, log, true);
   }
 
-  public Orchestrator(Config config, Optional<TopologyCatalog> topologyCatalog, Logger log) {
-    this(config, topologyCatalog, Optional.of(log));
+  public Orchestrator(Config config, Optional<TopologyCatalog> topologyCatalog, Optional<DagManager> dagManager, Logger log) {
+    this(config, topologyCatalog, dagManager, Optional.of(log));
   }
 
   public Orchestrator(Config config, Logger log) {
-    this(config, Optional.<TopologyCatalog>absent(), Optional.of(log));
+    this(config, Optional.<TopologyCatalog>absent(), Optional.<DagManager>absent(), Optional.of(log));
   }
 
   /** Constructor with no logging */
   public Orchestrator(Config config, Optional<TopologyCatalog> topologyCatalog) {
-    this(config, topologyCatalog, Optional.<Logger>absent());
+    this(config, topologyCatalog, Optional.<DagManager>absent(), Optional.<Logger>absent());
   }
 
   public Orchestrator(Config config) {
-    this(config, Optional.<TopologyCatalog>absent(), Optional.<Logger>absent());
+    this(config, Optional.<TopologyCatalog>absent(), Optional.<DagManager>absent(), Optional.<Logger>absent());
   }
 
   @VisibleForTesting
@@ -186,26 +199,72 @@ public class Orchestrator implements SpecCatalogListener, Instrumentable {
 
     long startTime = System.nanoTime();
     if (spec instanceof FlowSpec) {
-      Map<Spec, SpecExecutor> specExecutorInstanceMap = specCompiler.compileFlow(spec);
+      TimingEvent flowCompilationTimer = this.eventSubmitter.isPresent()
+          ? this.eventSubmitter.get().getTimingEvent(TimingEvent.FlowTimings.FLOW_COMPILED)
+          : null;
 
-      if (specExecutorInstanceMap.isEmpty()) {
+      Dag<JobExecutionPlan> jobExecutionPlanDag = specCompiler.compileFlow(spec);
+
+      Map<String, String> flowMetadata = TimingEventUtils.getFlowMetadata((FlowSpec) spec);
+      if (jobExecutionPlanDag == null || jobExecutionPlanDag.isEmpty()) {
+        // For scheduled flows, we do not insert the flowExecutionId into the FlowSpec. As a result, if the flow
+        // compilation fails (i.e. we are unable to find a path), the metadata will not have flowExecutionId.
+        // In this case, the current time is used as the flow executionId.
+        flowMetadata.putIfAbsent(TimingEvent.FlowEventConstants.FLOW_EXECUTION_ID_FIELD,
+            Long.toString(System.currentTimeMillis()));
+        TimingEvent flowCompileFailedTimer = this.eventSubmitter.isPresent() ? this.eventSubmitter.get()
+            .getTimingEvent(TimingEvent.FlowTimings.FLOW_COMPILE_FAILED) : null;
+        Instrumented.markMeter(this.flowOrchestrationFailedMeter);
         _log.warn("Cannot determine an executor to run on for Spec: " + spec);
+        if (flowCompileFailedTimer != null) {
+          flowCompileFailedTimer.stop(flowMetadata);
+        }
         return;
       }
 
-      // Schedule all compiled JobSpecs on their respective Executor
-      for (Map.Entry<Spec, SpecExecutor> specsToExecute : specExecutorInstanceMap.entrySet()) {
-        // Run this spec on selected executor
-        SpecProducer producer = null;
-        try {
-          producer = specsToExecute.getValue().getProducer().get();
-          Spec jobSpec = specsToExecute.getKey();
+      //If it is a scheduled flow (and hence, does not have flowExecutionId in the FlowSpec) and the flow compilation is successful,
+      // retrieve the flowExecutionId from the JobSpec.
+      flowMetadata.putIfAbsent(TimingEvent.FlowEventConstants.FLOW_EXECUTION_ID_FIELD,
+          jobExecutionPlanDag.getNodes().get(0).getValue().getJobSpec().getConfigAsProperties().getProperty(ConfigurationKeys.FLOW_EXECUTION_ID_KEY));
 
-          _log.info(String.format("Going to orchestrate JobSpec: %s on Executor: %s", jobSpec, producer));
-          producer.addSpec(jobSpec);
-        } catch(Exception e) {
-          _log.error("Cannot successfully setup spec: " + specsToExecute.getKey() + " on executor: " + producer +
-              " for flow: " + spec, e);
+      if (flowCompilationTimer != null) {
+        flowCompilationTimer.stop(flowMetadata);
+      }
+
+
+      if (this.dagManager.isPresent()) {
+        //Send the dag to the DagManager.
+        this.dagManager.get().offer(jobExecutionPlanDag);
+      } else {
+        // Schedule all compiled JobSpecs on their respective Executor
+        for (Dag.DagNode<JobExecutionPlan> dagNode : jobExecutionPlanDag.getNodes()) {
+          JobExecutionPlan jobExecutionPlan = dagNode.getValue();
+
+          // Run this spec on selected executor
+          SpecProducer producer = null;
+          try {
+            producer = jobExecutionPlan.getSpecExecutor().getProducer().get();
+            Spec jobSpec = jobExecutionPlan.getJobSpec();
+
+            if (!((JobSpec) jobSpec).getConfig().hasPath(ConfigurationKeys.FLOW_EXECUTION_ID_KEY)) {
+              _log.warn("JobSpec does not contain flowExecutionId.");
+            }
+
+            Map<String, String> jobMetadata = TimingEventUtils.getJobMetadata(flowMetadata, jobExecutionPlan);
+            _log.info(String.format("Going to orchestrate JobSpec: %s on Executor: %s", jobSpec, producer));
+
+            TimingEvent jobOrchestrationTimer = this.eventSubmitter.isPresent() ? this.eventSubmitter.get().
+                getTimingEvent(TimingEvent.LauncherTimings.JOB_ORCHESTRATED) : null;
+
+            producer.addSpec(jobSpec);
+
+            if (jobOrchestrationTimer != null) {
+              jobOrchestrationTimer.stop(jobMetadata);
+            }
+          } catch (Exception e) {
+            _log.error("Cannot successfully setup spec: " + jobExecutionPlan.getJobSpec() + " on executor: " + producer
+                + " for flow: " + spec, e);
+          }
         }
       }
     } else {
@@ -221,26 +280,27 @@ public class Orchestrator implements SpecCatalogListener, Instrumentable {
     // .. this will work for Identity compiler but not always for multi-hop.
     // Note: Current logic assumes compilation is consistent between all executions
     if (spec instanceof FlowSpec) {
-      Map<Spec, SpecExecutor> specExecutorInstanceMap = specCompiler.compileFlow(spec);
+      Dag<JobExecutionPlan> jobExecutionPlanDag = specCompiler.compileFlow(spec);
 
-      if (specExecutorInstanceMap.isEmpty()) {
+      if (jobExecutionPlanDag.isEmpty()) {
         _log.warn("Cannot determine an executor to delete Spec: " + spec);
         return;
       }
 
       // Delete all compiled JobSpecs on their respective Executor
-      for (Map.Entry<Spec, SpecExecutor> specsToDelete : specExecutorInstanceMap.entrySet()) {
+      for (Dag.DagNode<JobExecutionPlan> dagNode: jobExecutionPlanDag.getNodes()) {
+        JobExecutionPlan jobExecutionPlan = dagNode.getValue();
         // Delete this spec on selected executor
         SpecProducer producer = null;
         try {
-          producer = specsToDelete.getValue().getProducer().get();
-          Spec jobSpec = specsToDelete.getKey();
+          producer = jobExecutionPlan.getSpecExecutor().getProducer().get();
+          Spec jobSpec = jobExecutionPlan.getJobSpec();
 
           _log.info(String.format("Going to delete JobSpec: %s on Executor: %s", jobSpec, producer));
           producer.deleteSpec(jobSpec.getUri(), headers);
-        } catch(Exception e) {
-          _log.error("Cannot successfully delete spec: " + specsToDelete.getKey() + " on executor: " + producer +
-              " for flow: " + spec, e);
+        } catch (Exception e) {
+          _log.error("Cannot successfully delete spec: " + jobExecutionPlan.getJobSpec() + " on executor: " + producer
+              + " for flow: " + spec, e);
         }
       }
     } else {
